(:~
 : The bdn-search modules handles the search options for the BdN project.
 :
 : ATTENTION: This module hasn't been maintained since Juli 2016!
 :
 : @author Michelle Weidling
 :)

xquery version "3.1";

module namespace bdn-search="https://bdn-edition.de/ns/search";

declare namespace tei="http://www.tei-c.org/ns/1.0";
declare namespace templates="http://exist-db.org/xquery/templates";

import module namespace functx="http://www.functx.com";
import module namespace kwic="http://exist-db.org/xquery/kwic";
import module namespace console="http://exist-db.org/xquery/console";

declare
    %templates:wrap
    function bdn-search:main($node as node(), $model as map(*))
{
  <div>
      <div class="alert alert-warning" role="alert">
          Die Suche befindet sich derzeit noch im <b>Beta-Stadium</b>. Die volle Suchfunktionalität wird bis zum Erscheinen von Johann August Nösselts "Anweisung zur Bildung angehender Theologen" (<sup>1</sup>1786/89–<sup>3</sup>1818/19) zur Verfügung stehen.
      </div>

      <form action="result.html" class="form-inline">
          <div class="form-group">
              <div class="input-group" style="width: 600px;">
                  <input type="text" class="form-control" name="q" aria-describedby="basic-addon2" />
                  <span class="input-group-addon" id="bdn-search">
                  <button type="submit" class="btn btn-default" style="padding: 0px; line-height: 1; background: #eee; border: 0px; width:100%; height: 100%;"><i class="fa fa-search"></i></button>
                  </span>
              </div>
          </div>
      </form>

      <p style="margin-top: 1em;">
          Optionen für eine erweiterte Suche sowie Hinweise zur Benutzung finden Sie in der Seitenleiste.
      </p>
  </div>
};



declare
    %templates:wrap
    function bdn-search:hints($node as node(), $model as map(*))
{
  let $t1 := string('w1')
  let $t2 := string('w2')
  let $t3 := concat($t1, ' ', $t2)

  return
      <div>
          <table style="margin-top: 5px; margin-bottom: 10px;">
              <tr>
                  <td>
                      <em>{$t3}</em>
                  </td>
                  <td>
                      Suche nach <em>{$t1}</em> oder <em>{$t2}</em>
                  </td>
              </tr>
              <tr>
                  <td>
                      <em>{substring($t1,1,1)}*</em>
                  </td>
                  <td>
                      Suche nach <u>mehreren</u> unbekannten Buchstaben, z.B. <em>Wort</em> oder <em>Wolke</em>
                  </td>
              </tr>
              <tr>
                  <td>
                      <em>{substring($t1,1,1)}?</em>
                  </td>
                  <td>
                      Suche nach <u>einem</u> unbekannten Buchstaben, z.B. <em>{$t1}</em> oder <em>{$t2}</em>
                  </td>
              </tr>
              <tr>
                  <td>
                      <em>"{$t3}"</em>
                  </td>
                  <td>
                      Suche nach Phrase
                  </td>
              </tr>
              <tr>
                  <td style="width: 90px;">
                       <em>"{$t3}"~10</em>
                  </td>
                  <td>
                      <em>{$t1}</em> und <em>{$t2}</em> stehen nicht weiter als 10 Wörter auseinander
                  </td>
              </tr>
              <tr>
                  <td>
                      <em>{$t1}~</em>
                  </td>
                  <td>
                      unscharfe Suche
                  </td>
              </tr>
              <tr>
                  <td>
                      <em>+{$t3}</em>
                  </td>
                  <td>
                      muss {$t1} enthalten, kann {$t2} enthalten
                  </td>
              </tr>
              <tr>
                  <td>
                      <em>-{$t3}</em>
                  </td>
                  <td>
                      darf {$t1} nicht enthalten, kann {$t2} enthalten
                  </td>
              </tr>
          </table>
          Neben den genannten Optionen ist auch eine Suche mit Hilfe von Booleschen Operatoren (AND, OR, NOT) <!--sowie eine Gruppierung von Suchbegriffen mittels runder Klammern ()-->möglich.
      </div>
};


declare
    %templates:wrap
    function bdn-search:options($node as node(), $model as map(*))
{
  let $doc := doc("/db/sade/resources/full-xmls/noesselt-full.xml")

  let $author := $doc//tei:author
  let $title := $doc//tei:title[@level = 'a']/string()
  let $editions := string-join($doc//tei:witness/@xml:id/string(), " ")
  let $editions := distinct-values(tokenize($editions, "[#\s]+"))

  return
      <div>
          <div class="works" style="margin-bottom: 0.5em;">
          Werke <i class="fa fa-caret-down"></i>
              <div class="authors" style="margin-top: 0.5em; margin-left: 1em; display: none;">
                  {$author} <i class="fa fa-caret-down" />
                  <div class="opera" style="margin-left: 1em; display: none;">
                      <div class="checkbox" style="margin-left: 1em;">
                          <label>
                              <input type="checkbox" value="" />
                              {$title}
                          </label>
                          <div class="editions" style="margin-left: 1em;">
                              {
                              for $edition in $editions
                              where $edition != ""
                                      order by $edition
                                  return
                                      <div class="checkbox" style="margin-left: 1em;">
                                          <label>
                                                  <input type="checkbox" value="" />
                                                  {concat('Auflage ', $edition)}
                                          </label>
                                      </div>
                              }
                          </div>
                      </div>
                  </div>
              </div>
          </div>

          <div class="areas" style="margin-bottom: 0.5em;">
          Textbereiche <i class="fa fa-caret-down"></i>
              <div class="chapters" style="margin-top: 0.5em; margin-left: 1em; display: none;">
                  Kapitel <i class="fa fa-caret-down" />
                  <div class="chapter-selection" style="margin-left: 1em; display: none;">
                  hdfgjkhdfkjghkdj
                  </div>
              </div>
              <div class="other" style="margin-left: 1em; display: none;">
                  <div class="checkbox">
                      <label>
                          <input type="checkbox" id="headings" value="" />
                          Überschrift
                      </label>
                  </div>
                  <div class="checkbox">
                      <label>
                          <input type="checkbox" id="highlights" value="" />
                          Hervorhebung
                      </label>
                  </div>
                  <div class="checkbox">
                      <label>
                          <input type="checkbox" id="rdgs" value="" />
                          Textkritischer Apparat
                      </label>
                  </div>
                  <div class="checkbox">
                      <label>
                          <input type="checkbox" id="note-editorial" value="" />
                          Editorische Notiz oder Kommentar
                      </label>
                  </div>
              </div>
          </div>

          <div class="category" style="margin-bottom: 0.5em;">
          Kategorien <i class="fa fa-caret-down"></i>
              <div class="cat-opt" style="margin-top: 0.5em; margin-left: 1em; display: none;">
                  <div class="checkbox">
                      <label>
                          <input type="checkbox" id="persons" value="" />
                          Person
                      </label>
                  </div>
                  <div class="checkbox">
                      <label>
                          <input type="checkbox" id="bible" value="" />
                          Bibelstelle
                      </label>
                  </div>
              </div>
          </div>
      </div>
};

(: TOFIX: handling if q is empty :)
declare
    %templates:wrap
    function bdn-search:results($node as node(), $model as map(*))
{
    let $searchphrase := request:get-parameter("q", "")
    let $file := doc("/db/sade/resources/full-xmls/noesselt-full.xml")

    return
        if ($searchphrase = "")
            then    <div style="font-size: larger; margin-bottom: 30px; font-style: 'Lato', sans-serif;">
                        Bitte geben Sie einen Suchbegriff ein.
                    </div>
            else
        <div>
            <div style="font-size: larger; margin-bottom: 30px; font-style: 'Lato', sans-serif;">
                Suchergebnisse für "{$searchphrase}"
            </div>

        {

        if ($searchphrase = ())
            then ""
        else if ($file//*[ft:query(., $searchphrase)] != "")
            then ()
            else
                <div class="alert alert-warning" role="alert">
                    Leider war Ihre Suche nach <strong>"{$searchphrase}"</strong> nicht erfolgreich.

                    Bitte beachten Sie die rechts unter <strong>"Hinweise zur Benutzung"</strong> aufgeführten Suchoptionen                     und versuchen Sie es noch einmal.
                </div>
        }

        {
        (: alter: collection instead of doc :)
        for $text in $file//tei:div[@type = 'section']
            [ft:query(., $searchphrase)]

            let $parent-set := kwic:expand($text)//exist:match/ancestor::*[1]



                (:let $hits-in-rdg := console:log($hit/ancestor::tei:rdg):)
                (:let $hits-in-note := $hit/ancestor::tei:note[@type = 'editorial']:)


            let $chapter := $file//tei:div[@type = 'section'][descendant-or-self::tei:div = $text]/@n/string()

            let $navigation := local:get-navigation($file, $chapter, $parent-set)

        order by number($chapter) ascending
        return (
            <div style="margin-bottom: 50px; font-style: 'Lato', sans-serif;">

                {$navigation}

                <!--{for $hit in $text
                    let $navigation := local:get-navigation($file, $chapter, $hit)
                return

                        <div style="margin-bottom: 50px; font-style: 'Lato', sans-serif;">
                            {$navigation}

                            <div class="result">
                                {kwic:summarize($hit/ancestor::*[2], <config width="300" table="no" />,
                                util:function(xs:QName("local:filter"), 2))}
                            </div>

                        {console:log(kwic:expand($text)//exist:match)}
                        </div>
                }-->

                <div class="result">
                    {kwic:summarize($text, <config width="300" table="no" />,
                    util:function(xs:QName("local:filter"), 2))}
                </div>
            </div>

            )
        }

        <!--            <strong>tei:term</strong>
            {
            (: alter: collection instead of doc :)
            for $term in doc("/db/sade-projects/textgrid/data/xml/data/2501b.2.xml")//tei:term
                [ft:query(., $searchphrase)]
            let $chapter := doc("/db/sade-projects/textgrid/data/xml/data/2501b.2.xml")//tei:div[@type = 'section'][descendant::tei:term = $term]/@n/string()
            let $expanded := util:expand($term, "expand-xincludes=no")
            order by ft:score($term) descending
            return (
                $expanded//exist:match
            )
            }

            <strong>tei:persName</strong>
            {
            (: alter: collection instead of doc :)
            for $name in doc("/db/sade-projects/textgrid/data/xml/data/2501b.2.xml")//tei:persName
                [ft:query(., $searchphrase)]
                let $chapter := doc("/db/sade-projects/textgrid/data/xml/data/2501b.2.xml")//tei:div[@type = 'section'][descendant::tei:persName = $name]/@n/string()
            order by ft:score($name) descending
            return (
                string($chapter),
                kwic:summarize($name, <config width="60" table="no" />)
            )
            }-->

        </div>
};

(:
 : LOCAL FUNCTIONS
 :)

(:  http://stackoverflow.com/questions/21199812/ignored-xml-elements-show-up-near-exist-dbs-lucene-search-results :)

declare %private function local:filter($node as node(), $mode as xs:string) as xs:string?
{
  let $ignored-elements := doc('/db/system/config/db/sade-projects/textgrid/data/collection.xconf')//*:ignore/@qname/string()
  let $ignored-elements :=
    for $ignored-element in $ignored-elements
    let $ignored-element := substring-after($ignored-element, ':')
    return $ignored-element
  return
    if (local-name($node/parent::*) = ('term', 'rdg', $ignored-elements))
      then ()
    (:else if (local-name($node//ancestor::note[@type = 'editorial']) = ('note', $ignored-elements))
     then <strong>note</strong>:)
    else
      $node
};


declare %private function local:get-navigation($file as xs:string, $chapter as xs:string , $hit)
{
    let $searchphrase := request:get-parameter("q", ())
    let $file := doc("/db/sade/resources/full-xmls/noesselt-full.xml")
    let $title := $file//tei:title[@level = 'a']/string()
    let $author := $file//tei:titleStmt/tei:author/tei:persName/tei:ref/string()
    let $editions := string-join($file//tei:witness/@xml:id/string(), " ")
    let $editions := distinct-values(tokenize($editions, "[#\s]+"))
    let $url := "http://bdn-edition.de/book.html?"
    let $file-param := "file=noesselt-full"
    let $digital-param := "expand=true"
    let $key := "key="

    (:let $hit-ancestor := if (ancestor::tei:rdg)
                            then ancestor::tei:rdg
                        else if (ancestor::tei:note[@type = 'editorial'])
                            then ancestor::tei:note[@type = 'editorial']
                        else ():)

    (:for $iii in $parent
        let $bla := console:log($iii):)

    (:let $bla := if ($parent/parent::tei:note)
                    then console:log($parent/parent::tei:note/tei:label/text())
                else
                    console:log("NOT parent note"):)

    let $normed-key := if (contains($searchphrase, ' AND '))
            then replace($searchphrase, ' AND ', '+')
        else if (contains($searchphrase, ' OR '))
            then replace($searchphrase, ' OR ', '+')
        else if (contains($searchphrase, 'NOT'))
            then replace($searchphrase, 'NOT', '')
        else if (contains($searchphrase, ' '))
            then replace($searchphrase, ' ', '+')
        else $searchphrase
    let $hi-param := (:if (contains($normed-key, '*'))
        then $key || substring-before($normed-key, '*')
        else:)
            $key || $normed-key

    let $url-digital := $url || $hi-param || "&#38;" ||
                $digital-param  || "&#38;model=true" || "&#38;" ||
                $file-param || "#section_1_" || number($chapter)
    let $url-critical := $url || $hi-param || "&#38;model=true" ||
                "&#38;" || $file-param || "&#38;edition=app#section_1_"
                || number($chapter)


    return
      (
      <div>
          <div style="margin-bottom: 20px;">
              <div class="panel panel-default">
                  <div class="panel-heading">
                      <strong>{$author}.</strong> Anweisung zur Bildung angehender Theologen.
                  </div>
              </div>

              <div style="margin-bottom: 20px;">
                  <div class="btn-group" role="group" aria-label="Wählbare Ansichten">

                      <button type="button" class="btn btn-default disabled"
                              style="color: #3f7a8c; font-weight: bold; opacity: 1;">
                              {$chapter}
                      </button>

                      <a href="{$url-digital}" target="_blank">
                          <button type="button" class="btn btn-default help"
                                  data-toggle="tooltip" data-placement="top"
                                  title="Öffnet Textabschnitt {$chapter} in der interaktiven textkritischen Ansicht.">
                              interaktiv
                          </button>
                      </a>

                      <a href="{$url-critical}" target="_blank">
                          <button type="button" class="btn btn-default help"
                              data-toggle="tooltip" data-placement="top"
                              title="Öffnet Textabschnitt {$chapter} in der druckähnlichen textkritischen Ansicht.">
                          textkritisch
                          </button>
                      </a>

                      {
                          for $edition in $editions
                          let $url-sgl := $url || $hi-param || "&#38;"
                              || $file-param || "&#38;model=true" ||
                              "&#38;edition=" || $edition  || "#section_1_"
                              || number($chapter)

                          where $edition != ''
                          return (
                              <a href="{$url-sgl}" target="_blank">
                                  <button type="button" class="btn btn-default help"
                                  data-toggle="tooltip" data-placement="top"
                                  title="Öffnet Textabschnitt {$chapter} in Auflage {$edition}.">
                                      {concat($edition, ' ')}
                                  </button>
                              </a>)
                      }
                  </div>

                  <button type="button" class="btn btn-link help"
                      style="border: 0; opacity: 0.65; color: #3f7a8c; cursor: help;"
                      data-toggle="tooltip" data-placement="right"
                      title="Wählen Sie die gewünschte Ansicht für den Textabschnitt {$chapter} aus.">
                      <i class="fa fa-question-circle" aria-hidden="true"></i>
                  </button>

              </div>
          </div>
      </div>
      )
};

declare %private function local:get-editorial-notes () as xs:string?
{
  <div></div>
};

declare %private function local:get-readings($file as xs:string, $searchphrase as xs:string) as xs:string?
{
  <div>
    {$searchphrase}
  </div>
};


declare %private function local:get-index() as xs:string?
{
  <div></div>
};


declare
    %templates:wrap
    function bdn-search:main($node as node(), $model as map(*)) {

    let $query := request:get-parameter("q", "")
    let $authors := request:get-parameter("author", ())
    let $years := request:get-parameter("year", ())

    let $opts :=
        <options>
            <default-operator>and</default-operator>
        </options>

    let $hits := local:get-hits($query, $opts)
    let $faceted := local:facet-hits($hits, $authors, $years)

    return
        map
        {
            "hits": $hits,
            "faceted": $faceted,
            "authors": $authors,
            "years": $years
        }

};

declare
    %templates:wrap
    function bdn-search:results($node as node(), $model as map(*)) {

    let $xsl := doc("/db/apps/bibNeo/resources/xsl/results.xsl")

    let $parameters :=
        <parameters>
            <param name="query" value="{$model("query")}" />
            <param name="exist:stop-on-error" value="no" />
            <param name="exist:stop-on-warn" value="no" />
        </parameters>

    return transform:transform($model("faceted"), $xsl, $parameters)

};

declare
    %templates:wrap
    function bdn-search:count($node as node(), $model as map(*)) {

    count($model("faceted")//hit)

};

declare
    %templates:wrap
    function bdn-search:authors($node as node(), $model as map(*)) {

    local:build-facet($model("faceted")//author, $model("authors"), "author")

};

declare
    %templates:wrap
    function bdn-search:years($node as node(), $model as map(*)) {

    local:build-facet($model("faceted")//year, $model("years"), "year")

};


(:~
 : LOCAL FUNCTIONS: local:get-hits and local:facet-hits
 :)


declare function local:get-hits($query, $opts)
{
    let $collection := "/db/sade-projects/textgrid/data/xml/data"

    let $hits := collection($collection)//tei:TEI[ft:query(., $query, $opts)]

    let $hits :=
        <hits query="{$query}">
            {
                for $hit in $hits
                    return
                        <hit>
                            <file>{util:document-name($hit)}</file>
                        </hit>
            }
        </hits>

    let $hits :=
        <hits query="{$query}">
            {
                for $hit in $hits/hit
                    let $file := $hit/file/text()
                    let $latest := local:get-latest($hits, $file)

                    where $file = $latest

                    return
                        <hit>
                            <file>{$file}</file>
                        </hit>
            }
        </hits>

    return
        <hits query="{$query}">
            {
                for $hit in $hits/hit
                    let $file := $hit/file/text()
                    let $doc := doc($collection || "/" || $file)

                    order by local:get-score($doc, $query, $opts) descending

                    return
                        <hit>
                            <file>
                                {$file}
                            </file>
                            <title>
                                {local:get-title($doc)}
                            </title>
                            <kwics>
                                {local:get-kwics($doc, $query, $opts)}
                            </kwics>
                            <authors>
                                {local:get-authors($doc)}
                            </authors>
                            <years>
                                {local:get-years($doc)}
                            </years>
                        </hit>
            }
        </hits>

};

declare function local:facet-hits($hits, $authors, $years)
{
    let $hits :=
        if (count($authors)) then
            <hits query="{$hits/@query}">
                {
                    for $author in $hits//hit/authors/author/text()
                        where functx:is-value-in-sequence($author, $authors)
                        return $author/ancestor::hit
                }
            </hits>
        else
            $hits

    let $hits :=
        if (count($years)) then
            <hits query="{$hits/@query}">
                {
                    for $year in $hits//hit/years/year/text()
                        where functx:is-value-in-sequence($year, $years)
                        return $year/ancestor::hit
                }
            </hits>
        else
            $hits

    return
        <hits query="{$hits/@query}">
            {functx:distinct-deep($hits/hit)}
        </hits>

};


(:~
 : LOCAL FUNCTIONS: Functions to collect information on hits
 :)


declare function local:get-title($doc)
{
  $doc//tei:titleStmt/tei:title[1]/descendant-or-self::*/text()
};

declare function local:get-kwics($doc, $query, $opts)
{
    let $kwics :=
        for $hit in $doc//tei:TEI[ft:query(., $query, $opts)]
            return kwic:summarize($hit, <config width="40" />)

    for $kwic in $kwics[position() <= 3]
        return <kwic>{$kwic}</kwic>
};

declare function local:get-authors($doc)
{
    for $author in $doc//tei:titleStmt/tei:author/descendant-or-self::*/text()
        return <author>{$author}</author>
};

declare function local:get-years($doc)
{
  for $year in $doc//tei:sourceDesc/tei:biblStruct[1]//tei:date/descendant-or-self::*/text()
    return <year>{$year}</year>
};


(:~
 : LOCAL FUNCTIONS: Helper functions
 :)


declare function local:build-facet($all, $selected, $param)
{
    let $href := "search.html?" || request:get-query-string()

    for $text in distinct-values($all)
        where $text order by $text ascending

        return
            if (functx:is-value-in-sequence($text, $selected)) then
                let $encoded := encode-for-uri($text)
                let $replaced := replace($encoded, "%20", "+")
                let $replaced := replace($replaced, "%2C", ",")
                let $escaped := functx:escape-for-regex($replaced)
                let $search := "&#38;" || $param || "=" || $escaped
                let $href := replace($href, $search, "")

                return
                    <div>
                        <strong>{$text}</strong>
                        <span class="pull-right">
                            <a href="{$href}">
                                <i class="fa fa-minus-square-o" />
                            </a>
                        </span>
                    </div>
            else
                let $uri := replace($text, " ", "+")
                let $href := $href || "&#38;" || $param || "=" || $uri

                return
                    <div>
                        {$text}
                        <span class="pull-right">
                            <a href="{$href}">
                                <i class="fa fa-plus-square-o" />
                            </a>
                        </span>
                    </div>

};

declare function local:get-latest($hits, $file)
{
    let $uri := tokenize($file, "\.")[1]

    let $files :=
        for $file in $hits/hit[contains(file, $uri)]/file
            order by $file descending
            return $file

    return $files[1]

};

declare function local:get-score($doc, $query, $opts)
{
    let $score := 0

    for $hit in $doc//tei:TEI[ft:query(., $query, $opts)]
        let $score := $score + ft:score($hit)

    return $score

};
